SystemOrganization addCategory: #'FLOdactic-Courses'!

Object subclass: #CourseSimulator
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

!CourseSimulator methodsFor: 'simulating' stamp: 'afv 6/17/2013 12:11'!
simulateRoutineAlbert
	"Routine 1: A User creates a new course and adds subjects to it. Then delegates control of the course to another user"
	|admin mainUser topic course primeiroDia subject turmaA intro cm |
	
	admin := User new.
	admin username: 'user'; password: 'q'; name: 'Leonardo Santos'; email: 'admin@flodactic.org'; withAdminPrivileges; doEnable.

	mainUser := User new.
	mainUser username: 'user2'; password: 'q'; name: 'Leonardo Santos'; email: 'admin@flodactic.org'; withAdminPrivileges; doEnable.
	
		
	topic := CourseTopic withName: 'Computao' creator: admin.
	
	course := CourseCourse withName: 'BCC' creator: admin.
	course description: 'B.S. C.S.'.

	subject := CourseSubject new.
	subject initializeWithCode: 'MAC5414' initializeWithName: 'Introduo a Computao' creator: admin.
	subject parent: course.	

	cm := CoursesManager new.
	cm validate: course.
	cm owner: admin addCourse: course.
	
	intro := CourseLesson withName: 'Introducao' creator: admin.
	intro parent: subject.
	
	turmaA := CourseClass withName: 'TurmaA' creator: admin.
	turmaA parent: subject.
	
	primeiroDia := CourseLesson withName: 'Introducao' creator: admin.
	primeiroDia parent: turmaA.
	
	course explore.
	 ^ true! !

!CourseSimulator methodsFor: 'simulating' stamp: 'afv 6/6/2013 22:18'!
simulateRoutineOne
	"Routine 1: A User creates a new course and adds subjects to it. Then delegates control of the course to another user"
	|admin mainUser system topic course|
	
	admin := CourseUserRenamed withLogin: 'Pedros' password: 'senha' firstName: 'Pedro' lastName: 'Scocco'.
	mainUser := CourseUserRenamed withLogin: 'FabioKon' password: 'smalltalkS2' firstName: 'Fabio' lastName: 'Kon'.
	
 	system := CourseSystem withAdmin: admin.
 	topic := system createTopicWithName: 'Computao' creator: admin.
	
	course := topic createCourseWithName: 'BCC' creator: mainUser.
	
	course createSubjectWithName: 'Introduo a Computao' creator: mainUser.	
	
	course createSubjectWithName: 'Programao Orientada a Objetos' creator: mainUser.
	
	course addColaborator: admin.
	system explore.
	 ^ true! !

Object subclass: #CourseUnit
	instanceVariableNames: 'code name title description creationDate modificationDate parent children creator owner colaborators status modified tags'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

!CourseUnit commentStamp: 'DQM 5/12/2013 22:08' prior: 0!
'Base class for all classes in the course description hierarchy'!

CourseUnit subclass: #CourseClass
	instanceVariableNames: 'instructor users startDate endDate'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

!CourseClass commentStamp: 'DQM 5/12/2013 22:30' prior: 0!
'Class grouping multiple CourseLessons and participating CourseUsers'!

!CourseClass methodsFor: 'membership' stamp: ''!
addUser: user
    user = nil ifTrue: [
        self error: 'user cannot be nil'.
    ].

    users add: user.
    ^ self ! !

!CourseClass methodsFor: 'time' stamp: ''!
endDate
    ^ endDate ! !

!CourseClass methodsFor: 'time' stamp: ''!
endDate: date
    date = nil ifTrue: [
        self error: 'endDate cannot be nil'.
    ].

    date <= startDate ifTrue: [
        self error: 'endDate must succeed startDate'.
    ].

    endDate := date.
    ^ self ! !

!CourseClass methodsFor: 'initialization' stamp: 'PS 5/13/2013 19:01'!
initializeWithName: aName creator: aCreator creationDate: date
    super initializeWithName: aName creator: aCreator creationDate: date.
    
    users := Set new ! !

!CourseClass methodsFor: 'ownership and permissions' stamp: ''!
instructor
    ^ instructor ! !

!CourseClass methodsFor: 'ownership and permissions' stamp: ''!
instructor: anInstructor
    instructor := anInstructor.
    ^ self ! !

!CourseClass methodsFor: 'membership' stamp: ''!
removeUser: user
    user = nil ifTrue: [
        self error: 'user cannot be nil'.
    ].

    users remove: user ifAbsent: [].
    ^ self ! !

!CourseClass methodsFor: 'time' stamp: ''!
startDate
    ^ startDate ! !

!CourseClass methodsFor: 'time' stamp: ''!
startDate: date
    date = nil ifTrue: [
        self error: 'startDate cannot be nil'.
    ].

    date >= endDate ifTrue: [
        self error: 'startDate must precede endDate'.
    ].

    startDate := date.
    ^ self ! !

!CourseClass methodsFor: 'membership' stamp: ''!
users
    ^ users ! !

CourseUnit subclass: #CourseCourse
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

!CourseCourse commentStamp: 'DQM 5/12/2013 22:28' prior: 0!
'Course consisting of a series of CourseSubjects designed to teach a particular skillset'!

!CourseCourse methodsFor: 'creation' stamp: 'PS 5/13/2013 21:48'!
createSubjectWithName: aName creator: aCreator
	|newSubject|
	newSubject := CourseSubject withName: aName creator: aCreator.
	newSubject parent: self.
	^newSubject! !

!CourseCourse methodsFor: 'creation' stamp: 'it 6/21/2013 23:13'!
isCourse
	^ true! !

CourseUnit subclass: #CourseLesson
	instanceVariableNames: 'content'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

!CourseLesson commentStamp: 'DQM 5/12/2013 22:32' prior: 0!
'CourseLesson representing part of a Course through some form of Contents'!

!CourseLesson methodsFor: 'as yet unclassified' stamp: 'afv 6/17/2013 17:40'!
withName: aName withDidacticMaterial: aDidacticMaterial creator: aCreator

	self initializeWithName: aName creator: aCreator.! !

CourseUnit subclass: #CourseSubject
	instanceVariableNames: 'responsibleProfessor lessons'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

!CourseSubject commentStamp: 'DQM 5/12/2013 22:24' prior: 0!
'CourseSubject representing a series of lessons in an specific domain of a Course'!

!CourseSubject methodsFor: 'as yet unclassified' stamp: 'afv 6/19/2013 23:34'!
initializeWithCode2: aCodeString initializeWithName: aName creator: aCreator

	self initializeWithName: aName creator: aCreator.
	code := aCodeString.
	! !

!CourseSubject methodsFor: 'initialization' stamp: 'PS 5/13/2013 21:41'!
initializeWithName: aName creator: aCreator creationDate: date
    super initializeWithName: aName creator: aCreator creationDate: date.
    
    lessons := Set new ! !

!CourseSubject methodsFor: 'ownership and permissions' stamp: ''!
responsibleProfessor
    ^ responsibleProfessor ! !

!CourseSubject methodsFor: 'ownership and permissions' stamp: ''!
responsibleProfessor: professor
    responsibleProfessor := professor.
    ^ self ! !

CourseUnit subclass: #CourseSystem
	instanceVariableNames: 'admins users'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

!CourseSystem class methodsFor: 'instance creation' stamp: 'PS 5/13/2013 21:19'!
withAdmin: anAdmin
	^self basicNew 
		initializeWithAdmin: anAdmin creationDate: (DateAndTime now);
		yourself! !

!CourseSystem methodsFor: 'creation' stamp: 'DQM 5/13/2013 22:12'!
createTopicWithName: aName creator: aCreator
	|newTopic|
	newTopic := CourseTopic withName: aName creator: aCreator.
	newTopic parent: self.
	^ newTopic! !

!CourseSystem methodsFor: 'initialization' stamp: 'DQM 5/13/2013 22:10'!
initializeWithAdmin: anAdmin creationDate: aDate
	self initializeWithName: 'System' creator: anAdmin creationDate: aDate.

	admins := Set new add: anAdmin; yourself.
	users := Set new add: anAdmin; yourself.
	
	! !

CourseUnit subclass: #CourseTopic
	instanceVariableNames: 'moderators'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

!CourseTopic commentStamp: 'DQM 5/12/2013 22:26' prior: 0!
'CourseTopic representing a large field of knowledge, grouping multiple Courses'!

!CourseTopic methodsFor: 'ownership and permissions' stamp: ''!
addModerator: moderator
    moderator = nil ifTrue: [
        self error: 'moderator cannot be nil'.
    ].

    moderators add: moderator.
    ^ self ! !

!CourseTopic methodsFor: 'creation' stamp: 'PS 5/13/2013 21:48'!
createCourseWithName: aName creator: aCreator
	|newCourse|
	newCourse := CourseCourse withName: aName creator: aCreator.
	newCourse parent: self.
	^newCourse! !

!CourseTopic methodsFor: 'initialization' stamp: ''!
initializeWithName: aName creator: aCreator creationDate: date
    super initializeWithName: aName creator: aCreator creationDate: date.
    
    moderators := Set new ! !

!CourseTopic methodsFor: 'ownership and permissions' stamp: ''!
moderators
    ^ moderators ! !

!CourseTopic methodsFor: 'ownership and permissions' stamp: ''!
removeModerator: moderator
    moderator = nil ifTrue: [
        self error: 'moderator cannot be nil'.
    ].

    moderators remove: moderator ifAbsent: [].
    ^ self ! !

!CourseUnit class methodsFor: 'instance creation' stamp: 'PS 5/13/2013 21:23'!
withName: aName creator: aCreator
    ^ self withName: aName creator: aCreator creationDate: (DateAndTime now)! !

!CourseUnit class methodsFor: 'instance creation' stamp: 'PS 5/13/2013 21:23'!
withName: aName creator: aCreator creationDate: date
    ^ self basicNew
           initializeWithName: aName creator: aCreator creationDate: date;
           yourself ! !

!CourseUnit methodsFor: 'ownership and permissions' stamp: 'it 6/21/2013 22:44'!
addColaborator: aColaborator
	colaborators add: aColaborator name -> aColaborator.
	self touch.! !

!CourseUnit methodsFor: 'ownership and permissions' stamp: 'it 6/21/2013 22:47'!
addColaboratorByUsername: aUsername
	"Adds a user to the contributors, granting with editor permissions"
	
	| user  manager |
	manager := UsersManager singleton.
	
	"does it exist?"
	(manager items includesKey: aUsername) ifTrue: [
		user := manager items at: aUsername.
		colaborators add: aUsername name -> user ]! !

!CourseUnit methodsFor: 'time' stamp: 'it 6/21/2013 18:44'!
addTag: aTagString
	"Adds aTagString to the tags"

	| tag testIncludedTag tagsManager |
	
	tagsManager := TagsManager singleton.
	
	testIncludedTag := tagsManager includesKey: aTagString.
	"If it exists on the manager, get it"
	testIncludedTag ifTrue: [
		tag := tagsManager at: aTagString.
	]
	"else, add it to the manager"
	ifFalse: [
		tag := TagObject new.
		tag name: aTagString.
		tagsManager add: aTagString -> tag.
	].
	"finally add it to the tags object Dictionary itself"
	tags add: aTagString -> tag.! !

!CourseUnit methodsFor: 'hierarchy' stamp: ''!
children
    ^ children ! !

!CourseUnit methodsFor: 'as yet unclassified' stamp: 'afv 6/19/2013 23:24'!
code
	^ code.! !

!CourseUnit methodsFor: 'as yet unclassified' stamp: 'afv 6/19/2013 23:24'!
code: aCode
	code := aCode.! !

!CourseUnit methodsFor: 'ownership and permissions' stamp: ''!
colaborators
    ^ colaborators ! !

!CourseUnit methodsFor: 'time' stamp: ''!
creationDate
    ^ creationDate ! !

!CourseUnit methodsFor: 'time' stamp: ''!
creationDate: date
    date = nil ifTrue: [
        self error: 'creationDate cannot be nil'.
    ].

    creationDate := date.
    ^ self ! !

!CourseUnit methodsFor: 'ownership and permissions' stamp: ''!
creator
    ^ creator ! !

!CourseUnit methodsFor: 'ownership and permissions' stamp: ''!
creator: aCreator
    aCreator = nil ifTrue: [
        self error: 'creator cannot be nil'.
    ].

    creator := aCreator.

    ^ self ! !

!CourseUnit methodsFor: 'identification' stamp: ''!
description
    ^ description ! !

!CourseUnit methodsFor: 'identification' stamp: ''!
description: aDescription
    description := aDescription.
    ^ self ! !

!CourseUnit methodsFor: 'time' stamp: 'afv 6/11/2013 21:24'!
doDisable
	"Disables the didactic material. These type of messages are wrapped by another class who checks if the user can do that particular action."
	
	status := 'disabled'.
	self touch.! !

!CourseUnit methodsFor: 'time' stamp: 'afv 6/11/2013 21:20'!
doEnable
	"Enables the didactic material. These type of messages are wrapped by another class who checks if the user can do that particular action."

	status := 'enabled'.
	self touch.! !

!CourseUnit methodsFor: 'initialization' stamp: 'it 6/21/2013 22:53'!
initialize
	super initialize.
	children := Dictionary new.
	colaborators := Dictionary new.
	tags := Dictionary new! !

!CourseUnit methodsFor: 'as yet unclassified' stamp: 'afv 6/19/2013 23:34'!
initializeWithCode: aCodeString initializeWithName: aName creator: aCreator

	self initializeWithName: aName creator: aCreator.
	code := aCodeString.! !

!CourseUnit methodsFor: 'initialization' stamp: 'it 6/21/2013 18:47'!
initializeWithName: aName creator: aCreator
	self initializeWithName: aName creator: aCreator creationDate: (DateAndTime now)! !

!CourseUnit methodsFor: 'initialization' stamp: 'it 6/21/2013 22:53'!
initializeWithName: aName creator: aCreator creationDate: date
    self initialize.
    self name: aName;
         creator: aCreator;
         owner: aCreator;
         creationDate: date;
         modificationDate: date ! !

!CourseUnit methodsFor: 'hierarchy' stamp: 'it 6/21/2013 23:12'!
isCourse
    ^ false ! !

!CourseUnit methodsFor: 'time' stamp: 'afv 6/11/2013 21:24'!
isDisabled
	"Checks if the didactic material is disabled"
	
	^ status = 'disabled'! !

!CourseUnit methodsFor: 'time' stamp: 'afv 6/11/2013 21:25'!
isEnabled
	"Checks if the didactic material is enabled"
	
	^ status = 'enabled'! !

!CourseUnit methodsFor: 'time' stamp: ''!
modificationDate
    ^ modificationDate ! !

!CourseUnit methodsFor: 'time' stamp: ''!
modificationDate: date
    modificationDate := date.
    ^ self ! !

!CourseUnit methodsFor: 'identification' stamp: ''!
name
    ^ name ! !

!CourseUnit methodsFor: 'identification' stamp: ''!
name: aName
    aName = nil ifTrue: [
        self error: 'Name cannot be nil'.
    ].
    name := aName.
    ^ self ! !

!CourseUnit methodsFor: 'ownership and permissions' stamp: ''!
owner
    ^ owner ! !

!CourseUnit methodsFor: 'ownership and permissions' stamp: ''!
owner: anOwner
    anOwner = nil ifTrue: [
        owner := creator.
    ] ifFalse: [
        anOwner = owner ifFalse: [
            owner := anOwner.
        ].
    ].

    ^ self ! !

!CourseUnit methodsFor: 'hierarchy' stamp: ''!
parent
    ^ parent ! !

!CourseUnit methodsFor: 'hierarchy' stamp: 'it 6/21/2013 23:05'!
parent: aParent
	parent := aParent! !

!CourseUnit methodsFor: 'ownership and permissions' stamp: ''!
removeColaborator: colaborator
    colaborator = nil ifTrue: [
        self error: 'Colaborator cannot be nil'.
    ].

    colaborators remove: colaborator ifAbsent: [].
    ^ self ! !

!CourseUnit methodsFor: 'identification' stamp: 'it 6/21/2013 18:16'!
status
    ^ status! !

!CourseUnit methodsFor: 'identification' stamp: 'it 6/21/2013 18:42'!
tags
    ^ tags! !

!CourseUnit methodsFor: 'identification' stamp: ''!
title
    ^ title ! !

!CourseUnit methodsFor: 'identification' stamp: ''!
title: aTitle
    title := aTitle.
    ^ self ! !

!CourseUnit methodsFor: 'time' stamp: 'afv 6/11/2013 21:23'!
touch
	"Modifies the latest modification timestamp (similar to the touch *nix command)"

	modificationDate := DateAndTime now.! !

Object subclass: #CourseUserRenamed
	instanceVariableNames: 'login password firstName lastName createdUnits ownedUnits moderatedTopics participatingClasses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

!CourseUserRenamed class methodsFor: 'instance creation' stamp: ''!
withLogin: aLogin password: aPassword firstName: aFirstName lastName: aLastName
    ^ (self basicNew)
        initializeWithLogin: aLogin
        password: aPassword
        firstName: aFirstName
        lastName: aLastName;
        yourself ! !

!CourseUserRenamed methodsFor: 'identification' stamp: ''!
firstName
    ^ firstName ! !

!CourseUserRenamed methodsFor: 'identification' stamp: ''!
firstName: aFirstName   
    aFirstName isEmptyOrNil ifTrue: [
        self error: 'invalid firstName'.
    ].
    
    firstName := aFirstName.
    ^ self ! !

!CourseUserRenamed methodsFor: 'initialization' stamp: ''!
initializeWithLogin: aLogin password: aPassword firstName: aFirstName lastName: aLastName
    self initialize.

    self login: aLogin;
         password: aPassword;
         firstName: aFirstName;
         lastName: aLastName.

    createdUnits := Set new.
    moderatedTopics := Set new.
    participatingClasses := Set new ! !

!CourseUserRenamed methodsFor: 'identification' stamp: ''!
lastName
    ^ lastName ! !

!CourseUserRenamed methodsFor: 'identification' stamp: ''!
lastName: aLastName
    aLastName isEmptyOrNil ifTrue: [
        self error: 'invalid lastName'.
    ].
    
    lastName := aLastName.
    ^ self ! !

!CourseUserRenamed methodsFor: 'identification' stamp: ''!
login
    ^ login ! !

!CourseUserRenamed methodsFor: 'identification' stamp: ''!
login: aLogin
    (aLogin isEmptyOrNil or: [ aLogin size < 5 ]) ifTrue: [
        self error: 'invalid login'.
    ].

    login := aLogin.
    ^ self ! !

!CourseUserRenamed methodsFor: 'identification' stamp: ''!
password: aPassword
    password := UserPassword withPassword: aPassword.
    ^ self ! !

!CourseUserRenamed methodsFor: 'identification' stamp: ''!
verifyPassword: aPassword
    ^ password isNil ifTrue: [ false ] ifFalse: [ password verify: aPassword ] ! !

Object subclass: #CoursesManager
	instanceVariableNames: 'items status'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

CoursesManager class
	instanceVariableNames: 'uniqueInstance'!

CoursesManager class
	instanceVariableNames: 'uniqueInstance'!

!CoursesManager class methodsFor: 'as yet unclassified' stamp: 'afv 6/17/2013 12:15'!
singleton

	uniqueInstance ifNil: [
		Transcript show: 'Creating the singleton instance of CoursesManager...'; crlf.
		uniqueInstance := self basicNew initialize
	].
	^ uniqueInstance! !

!CoursesManager methodsFor: 'as yet unclassified' stamp: 'afv 6/9/2013 20:15'!
initialize
	"comment stating purpose of message"

	super initialize.
	items := Dictionary new.
	Transcript show: 'CourseManager initialize'; cr.! !

!CoursesManager methodsFor: 'as yet unclassified' stamp: 'it 6/21/2013 22:13'!
items
	"Returns a Dictionary with the items within the structure"

	^ items! !

!CoursesManager methodsFor: 'as yet unclassified' stamp: 'it 6/21/2013 23:29'!
owner: anOwner add: aCourse
	"comment stating purpose of message"
	| collection |
	self validate: aCourse.
	
	Transcript show: 'The course: *', aCourse name, '* DOESNT exists!!'; cr.
	aCourse owner: anOwner.
	aCourse doEnable.
	aCourse touch.
	
	aCourse isCourse
		ifTrue: [ collection := items ]
		ifFalse: [ collection := aCourse parent children ].
	
	collection add: aCourse code -> aCourse.! !

!CoursesManager methodsFor: 'as yet unclassified' stamp: 'it 6/21/2013 23:35'!
owner: anOwner edit: anItem
	"Edits a Course"
	| item collection |
	anItem isCourse
		ifTrue: [ collection := items ]
		ifFalse: [ collection := anItem parent children ].
	
	(collection includesKey: anItem code) ifFalse: [
		ValidationError signal: 'The course you are trying to update does not exist' ].
	
	self validate: anItem forExistingKey: anItem code.
	
	item := collection at: anItem code.
	item copyFrom: anItem.

	Transcript show: 'Editing: *', anItem code, '*'; cr.
	anItem touch.! !

!CoursesManager methodsFor: 'as yet unclassified' stamp: 'it 6/21/2013 23:36'!
remove: aCourse
	| collection |
	aCourse isCourse
		ifTrue: [ collection := items ]
		ifFalse: [ collection := aCourse parent children ].
		
	( collection includesKey: aCourse code )
		ifTrue: [ items removeKey: aCourse code ]
		ifFalse: [ self error: 'The course does not exists on the system, cannot remove it...' ]! !

!CoursesManager methodsFor: 'as yet unclassified' stamp: 'afv 6/17/2013 14:53'!
removeAll
	"Our dirty solution to avoid persistance in Squeak. In order to have the tests passing, we need to remove all the items on each test. This is not intended to be used by the API user"

	items keysDo: [:key | 
		Transcript show: 'Removing...', key printString; cr.
		items removeKey: key.
	].! !

!CoursesManager methodsFor: 'as yet unclassified' stamp: 'afv 6/19/2013 00:01'!
select: aFilter
	^ items select: aFilter! !

!CoursesManager methodsFor: 'as yet unclassified' stamp: 'it 6/21/2013 22:08'!
validate: aCourse
	"validates a course"
	self validate: aCourse forExistingKey: nil! !

!CoursesManager methodsFor: 'as yet unclassified' stamp: 'it 6/21/2013 23:29'!
validate: aCourse forExistingKey: aCode
	"validates a course"
	| collection |

	aCourse isCourse ifFalse: [
		(aCourse parent) ifNil: [
			ValidationError signal: 'A parent has to be defined!!' ] ].
	
	(aCourse code isEmptyOrNil)
		ifTrue: [
			ValidationError signal: 'Code cannot be empty!!'.
		] 
		ifFalse: [
			aCode ifNotNil: [
				aCourse isCourse
					ifTrue: [ collection := items ]
					ifFalse: [ collection := aCourse parent children ].
				
				(collection includesKey: (aCourse code)) ifTrue: [
					(aCourse code ~= aCode) ifTrue: [
						ValidationError signal: 'The code *', aCourse code, '* already exists!!' ] ] ] ].

	(aCourse description) ifNil: [
		ValidationError signal: 'Description cannot be empty!!' ].

	(aCourse title) ifNil: [
		ValidationError signal: 'Title cannot be empty!!' ]! !

Object subclass: #UserPassword
	instanceVariableNames: 'hash salt iterations'
	classVariableNames: 'Hasher Rng'
	poolDictionaries: ''
	category: 'FLOdactic-Courses'!

!UserPassword class methodsFor: 'initialization' stamp: ''!
initialize
    Hasher isNil ifTrue: [ Hasher := SecureHashAlgorithm new. ].
    Rng isNil ifTrue: [ Rng := Random new. ] ! !

!UserPassword class methodsFor: 'instance creation' stamp: ''!
withPassword: password
    UserPassword initialize.
    ^ (self basicNew) initializeWithPassword: password ! !

!UserPassword methodsFor: 'hashing and verification' stamp: ''!
hash: password
    | str |
    password = nil ifTrue: [
        self error: 'password cannot be nil'.
    ]. 

    str := password.
    iterations timesRepeat: [
        str := str, salt.
        str := self hashString: str.
    ].

    ^ str ! !

!UserPassword methodsFor: 'hashing and verification' stamp: ''!
hashString: str
    ^ ((Hasher hashMessage: str) printStringBase: 16) forceTo: 40 paddingStartWith: $0 ! !

!UserPassword methodsFor: 'initialization' stamp: ''!
initializeWithPassword: password
    self initializeWithPassword: password
         salt: (Hasher hashInteger: (Rng nextValue asInteger))
         iterations: 1000 ! !

!UserPassword methodsFor: 'initialization' stamp: ''!
initializeWithPassword: password salt: aSalt iterations: numIterations
    self initialize.

    iterations := numIterations.
    salt := self hashString: (aSalt asString).
    hash := (self hash: password) ! !

!UserPassword methodsFor: 'hashing and verification' stamp: 'DQM 5/13/2013 21:07'!
verify: password
    ^ hash isNil ifTrue: [ false ] ifFalse:  [ hash = (self hash: password) ]! !
UserPassword initialize!
